/* 
    Copyright (c) 2012 Microsoft Corporation.  All rights reserved.
    Use of this sample source code is subject to the terms of the Microsoft license 
    agreement under which you licensed this sample source code and is provided AS-IS.
    If you did not accept the terms of the license agreement, you are not authorized 
    to use this sample source code.  For the terms of the license, please see the 
    license agreement between you and Microsoft.
  
    To see all Code Samples for Windows Phone, visit http://go.microsoft.com/fwlink/?LinkID=219604 
  
*/
using System;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;
using sdkBluetoothA2DWP8CS.Resources;
using Windows.Networking.Proximity;
using Windows.Networking.Sockets;
using OpticonBluetoothAPI;

namespace sdkBluetoothA2DWP8CS
{
    public partial class MainPage : PhoneApplicationPage
    {
        private ObservableCollection<PairedDevice> _pairedDevices;  // a local copy of paired device information
        private BluetoothController _theAPI;

        // Constructor
        public MainPage()
        {
            InitializeComponent();
            Loaded += MainPage_Loaded;

            _theAPI = new BluetoothController();
            _theAPI.OnBarcodeReceived += _theAPI_OnBarcodeReceived;
            _theAPI.OnCommandStatusReceived += _theAPI_OnCommandStatusReceived;
        }

        void _theAPI_OnCommandStatusReceived(object sender, CommandStatusEventArgs e)
        {
            Dispatcher.BeginInvoke(() =>
                {
                    this.BarcodeDump.Text = e.Status.ToString() + " " + e.Command + " " + e.ReturnedData;
                });
        }

        void _theAPI_OnBarcodeReceived(object sender, BarcodeEventArgs e)
        {
            Dispatcher.BeginInvoke(() =>
                {
                    this.BarcodeDump.Text = e.Barcode;
                });
        }

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            // Bluetooth is not available in the emulator. 
            if (Microsoft.Devices.Environment.DeviceType == Microsoft.Devices.DeviceType.Emulator)
            {
                MessageBox.Show(AppResources.Msg_EmulatorMode,"Warning",MessageBoxButton.OK);
            }

            _pairedDevices = new ObservableCollection<PairedDevice>();
            PairedDevicesList.ItemsSource = _pairedDevices;
        }

        private void FindPairedDevices_Tap(object sender, GestureEventArgs e)
        {
            RefreshPairedDevicesList();
        }

        /// <summary>
        /// Asynchronous call to re-populate the ListBox of paired devices.
        /// </summary>
        private async void RefreshPairedDevicesList()
        {
            try
            {
                // Search for all paired devices
                List<PairedDevice> peers = await _theAPI.GetPairedDevices();

                // By clearing the backing data, we are effectively clearing the ListBox
                _pairedDevices.Clear();

                if (peers.Count == 0)
                {
                    MessageBox.Show(AppResources.Msg_NoPairedDevices);
                }
                else
                {
                    // Found paired devices.
                    foreach (PairedDevice peer in peers)
                    {
                        _pairedDevices.Add(peer);
                    }
                }
            }
            catch (InvalidOperationException ioe)
            {
                var result = MessageBox.Show(AppResources.Msg_BluetoothOff, "Bluetooth Off", MessageBoxButton.OKCancel);
                if (result == MessageBoxResult.OK)
                {
                    ShowBluetoothControlPanel();
                }
            }
            catch (Exception ex)
            {
                if ((uint)ex.HResult == 0x80070005)
                {
                    MessageBox.Show(AppResources.Msg_MissingCaps);
                }
                else
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void ConnectToSelected_Tap_1(object sender, GestureEventArgs e)
        {
            // Because I enable the ConnectToSelected button only if the user has
            // a device selected, I don't need to check here whether that is the case.

            // Connect to the device
            PairedDevice pdi = PairedDevicesList.SelectedItem as PairedDevice;

            // Asynchronous call to connect to the device
            ConnectToDevice(pdi);
        }

        private void ConnectToDevice(PairedDevice peer)
        {
            bool connected = _theAPI.ConnectToDevice(peer);

            if (!connected)
            {
                MessageBox.Show("Connection failed.");
            }
            else
            {
                ServiceNameInput.Visibility = Visibility.Collapsed;
                ConnectToSelected.IsEnabled = false;
                ConnectToSelected.Visibility = Visibility.Collapsed;
                FindPairedDevices.IsEnabled = false;
                FindPairedDevices.Visibility = Visibility.Collapsed;
                Z1Button.IsEnabled = true;
                Z1Button.Visibility = Visibility.Visible;
                DisconnectButton.IsEnabled = true;
                DisconnectButton.Visibility = Visibility.Visible;
            }

        }

        private void PairedDevicesList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // Check whether the user has selected a device
            if (PairedDevicesList.SelectedItem == null)
            {
                // No - hide these fields
                ConnectToSelected.IsEnabled = false;
                ServiceNameInput.Visibility = Visibility.Collapsed;
            }
            else
            {
                // Yes - enable the connect button
                ConnectToSelected.IsEnabled = true;

                // Show the service name field, if the ServiceName associated with this device is currently empty
                PairedDevice pdi = PairedDevicesList.SelectedItem as PairedDevice;
                ServiceNameInput.Visibility = (String.IsNullOrWhiteSpace(pdi.ServiceName)) ? Visibility.Visible : Visibility.Collapsed;
            }
        }

        private void ShowBluetoothControlPanel()
        {
            ConnectionSettingsTask connectionSettingsTask = new ConnectionSettingsTask();
            connectionSettingsTask.ConnectionSettingsType = ConnectionSettingsType.Bluetooth;
            connectionSettingsTask.Show();
        }

        private void DisconnectButton_Tap(object sender, GestureEventArgs e)
        {
            _theAPI.DisconnectFromDevice();
            FindPairedDevices.IsEnabled = true;
            FindPairedDevices.Visibility = Visibility.Visible;
            ServiceNameInput.Visibility = Visibility.Visible;
            ConnectToSelected.IsEnabled = true;
            ConnectToSelected.Visibility = Visibility.Visible;
            Z1Button.IsEnabled = false;
            Z1Button.Visibility = Visibility.Collapsed;
            DisconnectButton.IsEnabled = false;
            DisconnectButton.Visibility = Visibility.Collapsed;
        }

        private void Z1Button_Tap(object sender, GestureEventArgs e)
        {
            _theAPI.SendCommand("Z1");
        }
    }
}
